#define _CRT_SECURE_NO_WARNINGS
#include "ExtIO_SA.h"

#include "sa_api.h"
#pragma comment(lib,"sa_api.lib")

#include <thread>
#include <vector>
#include <iostream>
#include <atomic>

static int handle = -1;
static int iq_len = 0;
static double sps = 0;
static double bw = 0.0;
static int64_t center = 1.0e9;

pfnExtIOCallback pfnCallback;

std::vector<float> iq;
std::thread loopThread;

bool stopLoopThread = false;
static std::atomic<bool> reconfigure = false;

struct Atten {
    float displayValue;
};

static const int attenCount = 6;
static int attenIndex = 0;
Atten attenSettings[attenCount] = {
    { 0.0 },
    { 10.0 },
    { 20.0 },
    { 30.0 },
    { 40.0 },
    { 50.0 }
};

void loopRoutine()
{
    stopLoopThread = false;
    reconfigure = true;

    while(!stopLoopThread) {
        if(reconfigure) {
            saAbort(handle);
            saConfigLevel(handle, -attenSettings[attenIndex].displayValue);
            saConfigCenterSpan(handle, (double)center, 1.0e3);
            saInitiate(handle, SA_IQ, 0);
            reconfigure = false;
        }

        saGetIQ_32f(handle, &iq[0]);
        int ret = pfnCallback(iq_len, extHw_RUNNING, 0, &iq[0]);
    }
}

bool __stdcall InitHW(char *name, char *model, int &hwtype)
{
    sprintf(name, "Signal Hound");
    sprintf(model, "SA44/SA124");
    hwtype = 7;

    saStatus status = saOpenDevice(&handle);

    if(status != saNoError) {
        return false;
    }

    saConfigCenterSpan(handle, center, 1.0e3);
    saConfigGainAtten(handle, SA_AUTO_ATTEN, SA_AUTO_GAIN, true);
    saConfigIQ(handle, 1, 500.0e3);
    saConfigLevel(handle, -attenSettings[attenIndex].displayValue);

    saInitiate(handle, SA_IQ, 0);
    saQueryStreamInfo(handle, &iq_len, &bw, &sps);
    iq.resize(iq_len * 2);

    return true;
}

EXT_API bool __stdcall OpenHW()
{
    //loopThread = std::thread(&loopRoutine);
    pfnCallback(-1, extHw_READY, 0, NULL);

    return true;
}

EXT_API void __stdcall CloseHW()
{
    saCloseDevice(handle);
}

EXT_API int __stdcall StartHW64(int64_t extLOfreq)
{
    saStatus status = saInitiate(handle, SA_IQ, 0);
    if(status != saNoError) {
        return -1;
    }

    stopLoopThread = true;
    if(loopThread.joinable()) {
        loopThread.join();
    }
    saAbort(handle);
    loopThread = std::thread(&loopRoutine);

    return iq_len;
}

EXT_API void __stdcall StopHW()
{
    stopLoopThread = true;
    if(loopThread.joinable()) {
        loopThread.join();
    }
    saAbort(handle);
}

EXT_API void __stdcall SetCallback(pfnExtIOCallback funcptr)
{
    pfnCallback = funcptr;
}

EXT_API int __stdcall SetHWLO64(int64_t extLOfreq) // see also SetHWLO64
{
    if(extLOfreq < 1.0e3) {
        return -1;
    }

    if(extLOfreq > 4.4e9) {
        return 1;
    }

    center = extLOfreq;
    reconfigure = true;

    return 0;
}

EXT_API int __stdcall GetStatus()
{
    return 0;
}

EXT_API void __stdcall RawDataReady(long samprate, 
                                    void *Ldata, 
                                    void *Rdata, 
                                    int numsamples)
{
    int i = 0;
}

EXT_API long __stdcall GetHWLO()
{
    return center;
}

EXT_API long __stdcall GetHWSR()
{
    return sps;
}

EXT_API int __stdcall GetAttenuators(int idx, float *attenuation)
{
    if(idx >= attenCount) {
        return -1;
    }

    *attenuation = attenSettings[idx].displayValue;
    return 0;
}

EXT_API int __stdcall GetActualAttIdx()
{
    return attenIndex;
}

EXT_API int __stdcall SetAttenuator(int idx)
{
    if(idx >= attenCount || idx < 0) {
        return -1;
    }

    attenIndex = idx;
    reconfigure = true;
    return 0;
}

